      SUBROUTINE ice_bio_adv(kideb,kiut,nlay_i)

!------------------------------------------------------------------------------!
!
!                              --- ice_bio_adv ---
!
!    Gravity Drainage for tracers (Advection)
!
!------------------------------------------------------------------------------!

      USE lib_fortran
 
      INCLUDE 'type.com'
      INCLUDE 'para.com'
      INCLUDE 'const.com'
      INCLUDE 'ice.com'
      INCLUDE 'thermo.com'
      INCLUDE 'bio.com'

      INTEGER :: 
     &   ji                 ,    ! : horizontal space index
     &   jn                      ! : horizontal space index jn

      REAL(8), DIMENSION( maxnlay ) ::  !: dummy factors for tracer equation
     &   za                 ,    !: winter
     &   zb                 ,   
     &   zc                 ,   
     &   ze                 ,    !: summer
     &   zind               ,    !: independent term in the tridiag system
     &   zindtbis           ,    !:
     &   zdiagbis

      REAL(8), DIMENSION(maxnlay,3) ::!: dummy factors for tracer equation
     &   ztrid                   !: tridiagonal matrix

      REAL(8), DIMENSION(nlay_bio) ::  
     &   z_sbr_i                 !: brine salinity

      REAL(8), DIMENSION(ntra_bio_max) ::  
     &   zpp_gas                 !: partial pressure

      REAL(8) ::  
     &   zdummy1            ,    !: dummy factors
     &   zdummy2            ,    !: 
     &   zdummy3            ,    !: 
     &   zswitchs           ,    !: switch for summer drainage
     &   f_sn_rat           ,
     &   wspd_trs           ,           
     &   zsat_arg           ,
     &   zsat_oxy           ,
     &   zsat_CO2           ,
     &   zsat_nit           ,
     &   mol_diff

      INTEGER ::  
     &   indtr              ,    !: index of tridiagonal system
     &   iter                    !: time step  index

      REAL(8) ::
     &   int_DSI_n            ,
     &   int_DIN_n            ,
     &   int_DIP_n            ,
     &   int_DSI_n1           ,
     &   int_DIN_n1           ,
     &   int_DIP_n1
     
      LOGICAL ::
     &   ln_write_bio       ,
     &   ln_con_bio         ,
     &   ln_flood           

      ln_write_bio = .TRUE.
      ln_con_bio   = .TRUE.
      ln_flood     = .TRUE.

      zsol      = 0.
      zb0       = 0.
      zpatm_gas = 0. 
      zpp_gas(:) = 0.0

!=======================================================================

      WRITE(numout,*) 
      WRITE(numout,*) ' ** ice_bio_adv : '
      WRITE(numout,*) ' ~~~~~~~~~~~~~~~~~ '
      WRITE(numout,*)

      DO ji = kideb, kiut

!
!-----------------------------------------------------------------------
! 1) Initialization
!-----------------------------------------------------------------------
!
      IF ( ln_write_bio ) THEN
         WRITE(numout,*) ' Initialization '
         WRITE(numout,*) ' ~~~~~~~~~~~~~~ '
         WRITE(numout,*)
      ENDIF
      
      !---------------
      ! Interpolation 
      !---------------
      CALL ice_bio_interp_phy2bio(kideb,kiut,nlay_i,.FALSE.) 
                             ! interpolation of physical variables
                             ! on the biological grid
                             ! mass of salt, heat content, brine volume, Rb, PAR

      CALL ice_bio_interp_diffus(kideb,kiut,nlay_i,.TRUE.) 

      !--------------------------------
      ! Brine concentration of tracers
      !--------------------------------
      DO jn = 1, ntra_bio
         ! must implement adsorption here
         IF ( flag_diff(jn) .AND. flag_active(jn) ) THEN

            zf = 1 - f_ads(jn)
            c_i_bio(jn,:) = cbu_i_bio(jn,:) * zf / e_i_bio(:)

            WRITE(numout,*) ' 01 *** jn = ', jn
            WRITE(numout,*) ' c_i_bio : ', c_i_bio(jn,:)
         ENDIF
      END DO


      !--------------------------------
      ! Integrated old bulk concentration
      !--------------------------------

      ! jn = 1 pour => DSI
      ! jn = 2 pour => DIN
      ! jn = 3 pour => DIP

      int_DSI_n = SUM (cbu_i_bio(1,1:nlay_bio)*deltaz_i_bio(1:nlay_bio))
      int_DIN_n = SUM (cbu_i_bio(2,1:nlay_bio)*deltaz_i_bio(1:nlay_bio))
      int_DIP_n = SUM (cbu_i_bio(3,1:nlay_bio)*deltaz_i_bio(1:nlay_bio))

      WRITE(numout,*) ' int_DSI_n : ', int_DSI_n
      WRITE(numout,*) ' int_DIN_n : ', int_DIN_n
      WRITE(numout,*) ' int_DIP_n : ', int_DIP_n
      
      !---------------------------------------------------------------
      ! Equilibrate carbonate system to get aqueous CO2 concentration
      !---------------------------------------------------------------
!      IF ( ln_carbon ) CALL ice_carb_chem

!      DO jn = 1, ntra_bio
!         WRITE(numout,*) ' 02 *** jn = ', jn
!         WRITE(numout,*) ' c_i_bio : ', c_i_bio(jn,:)
!      END DO

      !--------------------
      ! Conservation check
      !--------------------

!      CALL ice_bio_column(kideb,kiut,ntra_bio,mt_i_bio_init,cbu_i_bio,
!     &                    deltaz_i_bio, .FALSE.)

!      IF ( ln_write_bio ) THEN
!         DO jn = 1, ntra_bio
!            WRITE(numout,*) ' mt_i_bio_init : ', mt_i_bio_init(jn)
!         END DO
!      ENDIF

      ! layer by layer
!      DO jn = 1, ntra_bio
!         DO jk = 1, nlay_bio
!            m_i_bio_init(jn,jk) = cbu_i_bio(jn,jk)*deltaz_i_bio(jk)
!         END DO
!      END DO
      

!------------------------------------------------------------------------------|
! Update ice concentrations
!------------------------------------------------------------------------------|
!
 
      DO jn = 1, ntra_bio
         IF ( flag_diff(jn) .AND. flag_active(jn) ) THEN

            DO layer = 1, nlay_bio
               za(layer) = w_adv_br(layer) * ddtb / deltaz_i_bio(layer)
            END DO

      ! first layer
            cbu_i_bio(jn,1) = c_i_bio(jn,1) * ( e_i_b(1) + za(1) ) +
     &                        c_i_bio(jn,2) * ( - za(1) )

      ! inner layers
            DO layer = 2, nlay_bio - 1
               cbu_i_bio(jn,layer) = c_i_bio(jn,layer-1) *
     &                               (za(layer)/2.) +
     &                               c_i_bio(jn,layer) * e_i_b(layer)  +
     &                               c_i_bio(jn,layer+1)*(-za(layer)/2.)
            END DO

         ! lowermost layer
            cbu_i_bio(jn,nlay_bio)  = c_i_bio(jn,nlay_bio-1) *
     &                                ( za(nlay_bio)/2. ) +
     &                                c_i_bio(jn,nlay_bio)   * 
     &                                (e_i_b(nlay_bio) +
     &                                za(nlay_bio)/2. ) - 
     &                                za(nlay_bio) * c_w_bio(jn)  

      IF ( ln_write_bio ) THEN
         WRITE(numout,*)
         WRITE(numout,*) ' cbu_i_bio   : ', ( cbu_i_bio(jn,layer) ,
     &                   layer = 1, nlay_bio )
      ENDIF

         ENDIF ! flag_diif flag_active
   
      END DO  ! ntrabio


!--------------------------------
! Integrated NEW bulk concentration
!--------------------------------

      ! jn = 1 pour => DSI
      ! jn = 2 pour => DIN
      ! jn = 3 pour => DIP
      
      int_DSI_n1 = SUM(cbu_i_bio(1,1:nlay_bio)*deltaz_i_bio(1:nlay_bio))
      int_DIN_n1 = SUM(cbu_i_bio(2,1:nlay_bio)*deltaz_i_bio(1:nlay_bio))
      int_DIP_n1 = SUM(cbu_i_bio(3,1:nlay_bio)*deltaz_i_bio(1:nlay_bio))

      WRITE(numout,*) ' int_DSI_n1 : ', int_DSI_n1
      WRITE(numout,*) ' int_DIN_n1 : ', int_DIN_n1
      WRITE(numout,*) ' int_DIP_n1 : ', int_DIP_n1

!-------------------------------------------------
! Flux nutrient with Sea Water (positive donward)
!-------------------------------------------------

      FDSI_AD = - (int_DSI_n1 - int_DSI_n)/ddtb
      FDIN_AD = - (int_DIN_n1 - int_DIN_n)/ddtb
      FDIP_AD = - (int_DIP_n1 - int_DIP_n)/ddtb

      WRITE(numout,*) ' FDIN_AD : ', FDIN_AD
      WRITE(numout,*) ' FDIP_AD : ', FDIP_AD
      WRITE(numout,*) ' FDSI_AD : ', FDSI_AD
     
      WRITE(numout,*) 'TESTBIO  w_adv_br : ', ( w_adv_br(layer),
     &                   layer = 1, nlay_bio )

!
!-----------------------------------------------------------------------
! 8) Mass of salt conserved ?
!-----------------------------------------------------------------------
!
      ! Final mass of salt
!      CALL ice_sal_column( kideb , kiut , z_ms_i_fin ,
!     &                    sn_i_b(1:nlay_i),
!     &                    deltaz_i_phy, nlay_i, .FALSE. )

      ! Bottom flux ( positive upwards for conservation routine )
!      zfb      =  - e_i_b(nlay_bio) *
!     &           ( diff_br(nlay_bio) * 2.0 / deltaz_i_bio(nlay_bio) *
!     &           ( c_i_bio(nlay_bio) - oce_sal ) + w_flood * ( z_flood *
!     &           oce_sal + ( 1. - z_flood ) * c_i_bio(nlay_bio) ) )
!     &           - e_i_b(nlay_bio) * w_flush * c_i_bio(nlay_bio)
!     &            - qsummer * z_sbr_i(nlay_i) / ddtb


!      fsb = - zfb * rhog / 1000. ! ice-ocean salt flux is positive downwards
!      IF ( ln_write ) THEN
!         WRITE(numout,*) ' fsb : ', fsb
!         WRITE(numout,*)
!      ENDIF

      ! Surface flux of salt
!      zfsu     = 0.0

      ! conservation check
!      zerror = 1.0e-15
!      CALL ice_sal_conserv(kideb,kiut,'ice_sal_adv  : ',zerror,
!     &                           z_ms_i_ini,z_ms_i_fin,
!     &                           zfb   , zfsu  , ddtb)

!      ENDIF ! ln_sal
!
!------------------------------------------------------------------------------|


!
!-----------------------------------------------------------------------
! 9) End of the routine
!-----------------------------------------------------------------------
!
      END DO ! ji

      IF ( ln_write_bio ) THEN

         WRITE(numout,*)
         WRITE(numout,*) ' *** After advection of tracers *** '
         WRITE(numout,*) '     model output '

         DO jn = 1, ntra_bio
            IF ( flag_active(jn) ) THEN
               WRITE(numout,*) ' biotr_i_nam : ', biotr_i_nam(jn)
               WRITE(numout,*) ' cbu_i_bio : ', ( cbu_i_bio(jn, jk), 
     &                         jk = 1, nlay_bio )
            ENDIF ! flag_active
         END DO ! jn
         WRITE(numout,*)

      ENDIF ! ln_write_bio

      IF ( ln_carbon ) CALL ice_carb_chem

      WRITE(numout,*)
      WRITE(numout,*) ' End of ice_bio_adv '
      WRITE(numout,*) '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'
!
!=============================================================================!
!-- End of ice_bio_adv --
 
      END SUBROUTINE
